local game = {}
love.graphics.setDefaultFilter('nearest', 'nearest')

spawn_x_lvl1, spawn_y_lvl1 = 36, 20
-- spawn_x_lvl1, spawn_y_lvl1 = 433, 350

screen1 = {alpha = 0, wait = 0,subtle_wait = 0}

--Teleporter animation
teleporterImage = love.graphics.newImage('img/TeleportAnim.png')
local g = anim8.newGrid(48, 48, teleporterImage:getWidth(), teleporterImage:getHeight())
animation = anim8.newAnimation(g('1-5', 1), 0.1, "pauseAtStart")

--Creates a world for items that the lasers can detect
laser_world1 = World.new()

local playerlvl1_has_teleported = false

--Instantiates lasers for level 1
local Lasers1 = require "lasers1"
local lasers = Lasers1.new()

--Creates an instance of the gameover screen for level 1
gameover = Gameover.new()

--imports and initiates map
bump_world1 = bump.newWorld(16)
local map = sti("map/map.lua", {"bump"})
map:bump_init(bump_world1)


--Instantiates a Player object and adds it to the collision detection world and laser world
player_lvl1 = Player.new(spawn_x_lvl1, spawn_y_lvl1, bump_world1)
bump_world1:add(player_lvl1, player_lvl1.x, player_lvl1.y, player_lvl1.w, player_lvl1.h)
laser_world1:add(player_lvl1)

--Requires the door and key for level one
local Doors1 = require "doors1"
local doors1 = Doors1.new()

function game:enter ()
  breath:play()
  screen1.alpha = 1.3
  const.level_complete = false

--Camera settings
  camera = Camera(player_lvl1.x, player_lvl1.y)
  camera.scale = 4
  camera:setDeadzone(const.ww/2, const.wh/4, 40,220)
  camera:setFollowLerp(0.02)
  camera:setBounds(-268,-174, 1000,870)
end

function game:update (dt)
  if const.get_out and playerlvl1_has_teleported == false then
    bump_world1:update(player_lvl1, 423, 430)
    playerlvl1_has_teleported = true
    player_lvl1.isDisabled = false
  end

  animation:update(dt)
  if const.get_out then
    camera:shake(5, 0.1, 20)
    camera:fade(0.5, {0.25, 0, 0, math.random(0.1, 0.4)})
  end
  camera:update(dt)
  camera:follow(player_lvl1.x, player_lvl1.y)

  if not run.timeout then
    for i, item in ipairs(laser_world1.items) do
      if item.update then
        item:update(dt)
      end
    end
  end

  if screen1.alpha > 0 and const.level_complete == false then
    screen1.alpha = screen1.alpha - 0.025
  end
  lasers:update(dt)
  doors1:update(dt)

  level1Done()
  gameover:update(dt)
  level1Transition()
end

function game:keypressed(key)
  --pauses game
  if key == 'p' and gameover.is_enabled == false and run.timeout == false then
    Gamestate.push(pause)
  end

  if key == 'return' and gameover.is_enabled == true then
    love.event.quit()
  end

  if key == 'r' and const.get_out == true then
    resetLevel1()
  end
end

function game:draw()

  love.graphics.setColor(1,1,1,1)
  camera:attach()
  love.graphics.draw(const.bg)
  animation:draw(teleporterImage, 16, 16)

  for i, item in ipairs(laser_world1.items) do
    if not const.get_out then
      item:draw()
    elseif item.isPlayer then
      item:draw()
    end
  end

  map:draw()
  camera:detach()
  camera:draw()

  gameover:draw()

  love.graphics.setColor(0,0,0,screen1.alpha)
  fade = love.graphics.rectangle('fill', 0, 0, const.ww, const.wh)
  run:draw()
end

--checks if player_lvl1 crosses the finish line
function level1Done ()
  if player_lvl1.x > 464 and const.level3_complete == false then
    player_lvl1.isDisabled = true
    const.level_complete = true
    const.level1_complete = true
  end
  if player_lvl1.x < 64 and player_lvl1.y < 64 and const.get_out == true then
    run.y = const.wh - 80
    breath:play()
    player_lvl1.isDisabled = true
    gameover.is_enabled = true
    run.paused = true
    const.get_out = false
    metronome:stop()
  end
end

--Fade in and out screen1
function level1Transition()
  if const.level1_complete == true and const.level3_complete == false then
    breath:play()
    screen1.subtle_wait = screen1.subtle_wait + 0.05
    if screen1.alpha < 1 and screen1.subtle_wait > 0.99 then
      screen1.alpha = screen1.alpha + 0.025
    end

    if screen1.alpha > 0.99 then
      if screen1.wait < 1 then
      screen1.wait = screen1.wait + 0.01
      end
    end
    if screen1.wait > 0.99 then
      const.level1_complete = false
      Gamestate.switch(level2)
    end
  end
end

--restarts get out sequence from after the artifact is acquired
function resetLevel1 ()
  player_lvl3.x = 611
  player_lvl3.y = 129
  bump_world3:update(player_lvl3, 611, 129)
  run.m = 3
  run.paused = false
  run.timeout = false
  const.level_complete = false
  const.level2_complete = false
  const.level3_complete = false
  player_lvl2.isDisabled = false
  player_lvl3.isDisabled = false
  screen2 = {alpha = 0, wait = 0, subtle_wait = 0}
  screen3 = {alpha = 0, wait = 0, subtle_wait = 0}
  player_lvl1.x = 414
  player_lvl1.y = 423
  bump_world1:update(player_lvl1, 414, 423)
  player_lvl2.x = 642
  player_lvl2.y = 401
  bump_world2:update(player_lvl2, 642, 401)
  Gamestate.switch(level3)
  level3:enter()
end


return game
