local pause = {}

local bg1 = love.graphics.newImage('img/bg2.png')
local bg2 = love.graphics.newImage('img/bg2.png')

local resume_img_cold = love.graphics.newImage("img/ResumeButton.png")
local resume_img_hot = love.graphics.newImage("img/ResumeButton2.png")

local quit_img_cold = love.graphics.newImage("img/QuitButton.png")
local quit_img_hot = love.graphics.newImage("img/QuitButton2.png")

local ww = love.graphics.getWidth()
local wh = love.graphics.getHeight()

local bg_x1, bg_y1 = 0,0
local bg_x2, bg_y2 = 0, -wh

local function newButton (name, img, fn)
  return {
    name = name,
    img = img,
    fn = fn,
    now = false,
    last = false
  }
end

local buttons = {}

table.insert(buttons, newButton("resume", resume_img_cold, function()
  print("Resuming game...")
  love.mouse.setPosition(-200,700)
  Gamestate.pop(pause)
end))
table.insert(buttons, newButton("quit", quit_img_cold, function()
  print("Quiting game...")
  love.event.quit()
end))

function pause:enter()
end

function pause:update(dt)
  bg_y1 = bg_y1 + 1
  bg_y2 = bg_y2 + 1
  if bg_y1 > wh then
    bg_y1 = 0
  end
  if bg_y2 > 0 then
    bg_y2 = -wh
  end
end


function pause:draw()
  local button_width = 312
  local button_height = 78
  local margin = 15
  local cursor_y = 0
  local total_height = (button_height + margin) * #buttons


  love.graphics.setColor(1,1,1,1)
  love.graphics.draw(bg1, bg_x1, bg_y1)
  love.graphics.draw(bg2, bg_x2, bg_y2)

  love.graphics.setColor(0,0,0,0.65)
  love.graphics.rectangle("fill", 0, 0, 720, 480)
  love.graphics.setColor(1,1,1,1)

  for i, button in ipairs(buttons) do
    button.last = button.now
    local bx = (ww * 0.5) - (button_width * 0.5)
    local by = (wh * 0.5) - (total_height * 0.5) + cursor_y

    local mx, my = love.mouse.getPosition()
    local hot = mx > bx and mx < bx + button_width and
                my > by and my < by + button_height

    if hot then
      if button.name == "resume" then
        button.img = resume_img_hot
      elseif button.name == "quit" then
        button.img = quit_img_hot
      end
    end

    if not hot then
      if button.name == "resume" then
        button.img = resume_img_cold
      elseif button.name == "quit" then
        button.img = quit_img_cold
      end
    end

    button.now = love.mouse.isDown(1)
    if button.now and not button.last and hot then
      button.fn()
    end

    love.graphics.draw(button.img, bx, by)
    cursor_y = cursor_y + (button_height + margin)
  end
end


function pause:keypressed(key)
  if key == "q" then
    love.event.quit()
  end
  if key == 'p' then
    Gamestate.pop(pause)
  end
end

return pause
